@@ -112,6 +112,7 @@ def gen_zonal_stats(
         with zonal stats appended as additional properties.
         Use with `prefix` to ensure unique and meaningful property names.
 
+        
     Returns
     -------
     generator of dicts (if geojson_out is False)
@@ -187,7 +188,11 @@ def gen_zonal_stats(
                     raise TypeError(('zone_func must be a callable '
                                      'which accepts function a '
                                      'single `zone_array` arg.'))
-                zone_func(masked)
+                value = zone_func(masked)
+
+                # check if zone_func has return statement
+                if value is not None:
+                    masked = value
 
             if masked.compressed().size == 0:
                 # nothing here, fill with None and move on
@@ -197,8 +202,12 @@ def gen_zonal_stats(
             else:
                 if run_count:
                     keys, counts = np.unique(masked.compressed(), return_counts=True)
-                    pixel_count = dict(zip([np.asscalar(k) for k in keys],
-                                           [np.asscalar(c) for c in counts]))
+                    try:
+                        pixel_count = dict(zip([k.item() for k in keys],
+                                               [c.item() for c in counts]))
+                    except AttributeError:
+                        pixel_count = dict(zip([np.asscalar(k) for k in keys],
+                                               [np.asscalar(c) for c in counts]))
 
                 if categorical:
                     feature_stats = dict(pixel_count)
@@ -254,7 +263,11 @@ def gen_zonal_stats(
 
             if add_stats is not None:
                 for stat_name, stat_func in add_stats.items():
-                    feature_stats[stat_name] = stat_func(masked)
+                    try:
+                        feature_stats[stat_name] = stat_func(masked, feat['properties'])
+                    except TypeError:
+                        # backwards compatible with single-argument function
+                        feature_stats[stat_name] = stat_func(masked)
 
             if raster_out:
                 feature_stats['mini_raster_array'] = masked
